<?php

PHPTAL::setIncludePath();
require_once "PHPTAL/PreFilter.php";
PHPTAL::restoreIncludePath();

class PHPTAL_Compress extends PHPTAL_PreFilter 
{
    function filterDOM(PHPTAL_Dom_Element $root)
    {
        $this->compressElement($root, $root, $root);
    }
    
    const MODE_BODY_BLOCK = 0;
    const MODE_BODY_INLINE = 0;
    const MODE_HEAD = 1;
    const MODE_PRE = 2;
    
    
    private function compressElement(PHPTAL_Dom_Element $element, PHPTAL_Dom_Node $preceeding_node, PHPTAL_Dom_Node $following_node)
    {        
        // don't compress tal:block attributes, etc.
        if (!$element->getXmlnsState()->isHandledNamespace($element->getNamespaceURI())) {
            $this->compressAttributes($element); 
        }
        
        if ($element->getAttributeNS("http://www.w3.org/XML/1998/namespace",'space') == 'preserve') {
            return;
        }
        
        if (count($element->childNodes)) {
            $i=0;
            $node = $preceeding_node;
            $nextnode = $element->childNodes[0];            
            do {
                $prevnode = $node;
                $node = $nextnode;
                $nextnode = (++$i < count($element->childNodes)) ? $element->childNodes[$i] : $following_node;
                
                if ($node instanceof PHPTAL_Dom_Element) {
                    $this->compressElement($node, $prevnode, $nextnode);
                }
                else if ($node instanceof PHPTAL_Dom_Text || $node instanceof PHPTAL_Dom_CDATASection) {
                    $val = $node->getValueEscaped();
                    
                    if (!$this->isInlineElement($prevnode)) {
                        $val = ltrim($val);
                    }
                    if (!$this->isInlineElement($nextnode)) {
                        $val = rtrim($val);
                    }
                    
                    $node->setValueEscaped($val);
                }
                
            } while($i < count($element->childNodes));
        }
    }
    
    private function isInlineElement(PHPTAL_Dom_Node $node)
    {
        if (!$node instanceof PHPTAL_Dom_Element) return true;
        if ($node->getNamespaceURI() !== 'http://www.w3.org/1999/xhtml' && $node->getNamespaceURI() === '') return false;
        
        return in_array($node->getLocalName(), array('a','span','i','b'));
    }
    
    private function compressAttributes(PHPTAL_Dom_Element $element)
    {
        $attrs_by_qname = array();
        
        foreach($element->getAttributeNodes() as $attrnode)
        {
            // skip attributes that are in a namespace (XHTML attributes have no namespace!)
            // skip replaced attributes (because getValueEscaped on them is meaningless)
            if ($attrnode->getNamespaceURI() !== ''
             || $attrnode->getReplacedState() !== PHPTAL_Dom_Attr::NOT_REPLACED) {
                continue;
            }
                        
            $val = $attrnode->getValueEscaped();            
            $val = preg_replace($attrnode->getEncoding()==='UTF-8' ? '/\s+/u' : '/\s+/', ' ',trim($val));
            
            $attrnode->setValueEscaped($val);
            
            $attrs_by_qname[$attrnode->getQualifiedName()] = $attrnode; // there can't be two attrs with same qname
        }
        
        // sorting of attributes might give slightly better gzip performance
        if (count($attrs_by_qname) > 1) {
            uksort($attrs_by_qname, array($this,'compareQNames'));        
            $element->setAttributeNodes(array_values($attrs_by_qname));
        }
    }
    
    /**
     * pre-defined order of attributes roughly by popularity
     */
    private static $attributes_order = array('href','src','class','title','width','height','name','style','lang','id','rel');
    
    /**
     * compare names according to $attributes_order array. 
     * Elements that are not in array, are considered greater than all elements in array,
     * and are sorted alphabetically.
     */
    private function compareQNames($a, $b)
    {
        $a_index = array_search($a, self::$attributes_order);
        $b_index = array_search($b, self::$attributes_order);
        
        if ($a !== false && $b !== false)
        {
            return $a - $b;
        }
        if ($a === false && $b === false)
        {
            return strcmp($a,$b);
        }
        return ($a === false) ? 1 : -1;
    }
    
    private function useless()
    {
        
        if (trim($element->getAttributeNS("http://www.w3.org/XML/1998/namespace",'space')) == 'preserve') {
            $mode = self::MODE_PRE;
        }
        else if ($element->getNamespaceURI() === 'http://www.w3.org/1999/xhtml' 
              || $element->getNamespaceURI() === '') {
            switch($element->getLocalName()) {
                case 'pre': case 'script': // script is newline-sensitive
                    $mode = self::MODE_PRE;
                    break;
                case 'html': case 'head':
                case 'table': case 'tr': case 'thead': case 'tbody': // inter-table can be cleaned up
                    $mode = self::MODE_HEAD;
                    break;
                case 'body': case 'div':  // body should be enough in valid documents
                case 'td': case 'th': // table changes mode
                case 'style': // styles can be safely wrapped
                    $mode = self::MODE_BODY_BLOCK;
                    break;
                case 'span': case 'a':
                    $mode = self::MODE_BODY_INLINE;
                    break;
            }                
        }
    }
}
